from fast_nosql_manager.interfaces.db_config_interface import DBConfigInterface
from pymongo import MongoClient, errors
from pymongo import errors

class Mongo(object):
  """ 
    Os IFs iniciáis são checagens de tipo
    para que não seja possível a quebra das
    funções
  """

  def __init__(self, db_config: DBConfigInterface, db_name: str = 'mongo'):
    self._conn: MongoClient = db_config.get_connection()
    self._db_name = db_name
    
  def delete_document(self, collection_name, where):
    collection = self._conn[self._db_name][collection_name]
    return collection.delete_one(where)
    
  def create_document(self, collection_name, documents):
    collection = self._conn[self._db_name][collection_name]
    
    if isinstance(documents, list):
      return collection.insert_many(documents)
  
    return collection.insert_one(documents)
    
  def delete_collection(self, collection_name):
    try:
      return self._conn[self._db_name].drop_collection(collection_name)
    except errors.CollectionInvalid:
      return
    
  def create_collection(self, collection_name: str):
    try:
      return self._conn[self._db_name].create_collection(collection_name)
    except errors.CollectionInvalid:
      return
      
  def select_all(self, collection_name, where={}):
    return_data=[]
    collection = self._conn[self._db_name][collection_name]
    for item in collection.find(where): return_data.append(item)
    return return_data


from fast_nosql_manager.abstractions._mongo import Mongo
from fast_nosql_manager.interfaces.db_config_interface import DBConfigInterface
import pymongo


class DataBaseConfig(DBConfigInterface):
  """ 
  Inicialização da classe de conexão
  ao banco
  """

  def __init__(self, db_str_connection: str, db_name: str = 'mongo'):
    self._db_connection = db_str_connection
    self.name = db_name


  def get_connection(self):
    mydb = pymongo.MongoClient(self._db_connection, serverSelectionTimeoutMS=5000)
    return mydb


class MongoRepository(Mongo):
  def __init__(self, db_str_connection: str, db_name: str):
    super().__init__(db_config=DataBaseConfig(db_str_connection), db_name=db_name)



from .implementations.mongo import MongoRepository

mongo = MongoRepository(
  db_str_connection='mongodb://localhost:27017/',
  db_name='local'
) 

def test_create_collection():
  mongo.create_collection('teste')
  
def test_create_document():
  mongo.create_document('teste', [{'name': 'Oscar'}, {'name': 'Oscar'}])
  
def test_select_all():
  response = mongo.select_all(collection_name='teste')
  assert len(response) == 2
  
def test_delete_document():
  mongo.delete_document('teste', {'name': 'Oscar'})
  response = mongo.select_all(collection_name='teste')
  assert len(response) == 1

def test_delete_collection():
  mongo.delete_collection('teste')